@@ -1,7 +1,7 @@
 
 /* 
  Authors
- Martin Schlather, martin.schlather@cu.lu
+ Martin Schlather, schlath@hsu-hh.de
 
  all around the nugget effect -- needs special treatment 
 
@@ -27,10 +27,11 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include <stdlib.h>
 #include <assert.h>
 #include "RFsimu.h"
+#include "RFCovFcts.h"
 
 
-static Real NUGGET_TOL_SINGULAR=1e-12;
-static Real NUGGET_TOL_POINTS=1e-12;
+static double NUGGET_TOL_SINGULAR=1e-12;
+static double NUGGET_TOL_POINTS=1e-12;
 
 typedef struct nugget_storage {
   double sqrtnugget;
@@ -50,13 +51,9 @@ void nugget_destruct(void ** S)
 }
 
 
-/* nugget effect model */
-Real nugget(Real *x, Real *p, int dim){
-  if (*x==0.0) return 1.0;  return 0.0;
-}
-Real Scalenugget(Real *p, int scaling) { return 1.0; }//or better 0.0 => error?
 
-int is_singular(Real *C, int dim, bool *singular)
+/* nugget effect model */
+int is_singular(double *C, int dim, bool *singular)
 {
   double D[MAXDIM];
   int Xerror, d;
@@ -70,9 +67,9 @@ int is_singular(Real *C, int dim, bool *singular)
   return Xerror;
 }
 
-bool equal(int i, int j, Real *ORDERD, int ORDERDIM)
+bool equal(int i, int j, double *ORDERD, int ORDERDIM)
 {
-  Real *x, *y;
+  double *x, *y;
   int d;
   x = ORDERD + i * ORDERDIM;
   y = ORDERD + j * ORDERDIM;
@@ -92,17 +89,98 @@ int init_nugget(key_type *key, int m){
   int Xerror; 
   nugget_storage *s;
   param_type param;
-  Real nugget_effect, quotient[MAXCOV];
+  double nugget_effect, quotient[MAXCOV];
   unsigned short int actcov;
   int i,covnr[MAXCOV], nonzero_pos;
-  int multiply[MAXCOV], TrueDim;
+  int multiply[MAXCOV], TrueDim, v;
   bool singular, no_last_comp;
-  Real *xx;
+  double *xx;
 
   xx=NULL;
   nonzero_pos = -1;
   SET_DESTRUCT(nugget_destruct);
-  FIRSTCHECK_COV_ANISO(Nugget, cov, param, false);
+  
+  double store_param[TOTAL_PARAM];
+  long bytes;
+  /* if (key->Time) bytes = key->timespacedim * (key->timespacedim-1); */
+  /* else */
+  bytes = sizeof(double) * key->timespacedim * key->timespacedim;
+
+  actcov=0;
+  for (v=0; v<key->ncov; v++) {
+    if ((key->method[v]==Nugget) && (key->left[v])) {
+      //Variance==0 is not eliminated anymore !!! -- maybe this could be improved
+      //	&& (key->param[v][VARIANCE]>0)) {   
+      // do not remove the parenths around METHOD!
+      key->left[v] = false;
+      assert((key->covnr[v] >= 0) && (key->covnr[v] < currentNrCov));
+      assert(key->param[v][VARIANCE] >= 0.0);
+      covnr[actcov] = key->covnr[v];
+      if (CovList[covnr[actcov]].implemented[Nugget] != IMPLEMENTED) {
+      //  formerly:   if (CovList[covnr[actcov]].cov==NULL) {
+	Xerror=ERRORNOTDEFINED; goto ErrorHandling;}
+      memcpy(param[actcov], key->param[v], sizeof(double) * key->totalparam);
+      if (actcov>0) { 
+        // actcov>0 not v>0 since next line actcov-1 -- check not for all v
+	if ((multiply[actcov-1] = key->op[v-1]) && key->method[v-1] != Nugget){
+          if (GENERAL_PRINTLEVEL>0) 
+	    PRINTF("severe error - contact author. %d %d %d %d (%s) %d (%s)n",
+		   v, key->op[v-1], key->ncov, key->method[v-1],
+		   METHODNAMES[key->method[v-1]],
+		   Nugget, METHODNAMES[Nugget]);
+          Xerror=ERRORMETHODMIX; goto ErrorHandling;
+	}
+      }
+
+      if (key->anisotropy) {
+	bool equal;
+	int w, endfor;
+	endfor = key->totalparam;
+	// if (key->Time && TIMEEXCEPTION) endfor -= key->timespacedim;
+	double f_epsilon = 1e-15;
+	if (actcov>0) {
+	  /* check whether the multiplicative construction are consistent, 
+	     i.e. that the SPATIAL part of the anisotropy matrices are 
+	     multiplicatives of each other (more precisely, the remaining ones
+	     are multiplicatives of the first.) 
+	     The time part of the matrix must be of the form (0,..,0,*).
+	     A further goal of this part is the collection of additive blocks 
+	     that have the same anisotropy matrix structure, in order to minimise
+	     the simulation time */
+	  assert(nonzero_pos>0);
+	  quotient[actcov] = param[actcov][nonzero_pos]/store_param[nonzero_pos];
+	  equal = true;
+	  for (w=ANISO; w<endfor; w++) {
+	    equal &= fabs(store_param[w] * quotient[actcov]
+		      - param[actcov][w]) <
+              (fabs(param[actcov][w]) + 1.0*(param[actcov][w]==0.0)) *f_epsilon;
+	  }
+	  if (!equal) { /* even not equal up to a multiplicative constant */
+	    if (multiply[actcov-1]) { Xerror=ERRORANISOMIX; goto ErrorHandling; }
+	    else { /* ???? nur additive ********* */
+	      key->left[v]=true;
+	      actcov--;
+	    }
+	  }
+	} else {
+	  memcpy(&(store_param[ANISO]), &(param[actcov][ANISO]), bytes);
+	  nonzero_pos=ANISO;
+	  quotient[0] = 1.0;
+	  while ((nonzero_pos<endfor) && (param[actcov][nonzero_pos]==0))
+	    nonzero_pos++;
+	  if (nonzero_pos>=endfor) { Xerror=ERRORTRIVIAL; goto ErrorHandling; }
+	}
+      } else {
+	assert(fabs(key->param[v][SCALE] * key->param[v][INVSCALE]-1.0)
+	       < EPSILON);
+      }
+      actcov++;
+    }
+  }
+  if (actcov==0) { /* no covariance for the considered method found */
+    Xerror=NOERROR_ENDOFLIST;
+    goto ErrorHandling;
+  }
 
   if ((key->S[m]=malloc(sizeof(mpp_storage)))==0){
     Xerror=ERRORMEMORYALLOCATION; goto ErrorHandling;
@@ -150,14 +228,16 @@ int init_nugget(key_type *key, int m){
   return Xerror;
 }
 
-void do_nugget(key_type *key, bool add, int m, Real *res ) {
-  Real sqrtnugget;
+void do_nugget(key_type *key, bool add, int m, double *res ) {
+  double sqrtnugget;
   nugget_storage* s;
   long nx, endfor;
+
 #define RESULT(OP)\
   for (nx=0,endfor=key->totalpoints; nx<endfor; nx++)\
      res[nx] OP GAUSS_RANDOM(sqrtnugget);\
 
+  assert(key->S[m] != NULL);
   s = (nugget_storage*) key->S[m];
   sqrtnugget = s->sqrtnugget;
 
@@ -165,7 +245,7 @@ void do_nugget(key_type *key, bool add, int m, Real *res ) {
     if (add) { RESULT(+=) } else { RESULT(=) } 
   } else {
     int p;
-    Real dummy;
+    double dummy;
     assert(s->pos[0]>=0);
     dummy = RF_NAN;
     for (nx=0; nx<key->totalpoints; nx++) {
@@ -178,6 +258,3 @@ void do_nugget(key_type *key, bool add, int m, Real *res ) {
   }
 }
 
-void rangenugget(int dim, int *index, Real* range){
-  *index = -1;
-}
