@@ -1,7 +1,6 @@
-
 /*
  Authors
- Martin Schlather, martin.schlather@cu.lu
+ Martin Schlather, schlath@hsu-hh.de
 
  Simulation of a random field by turning bands;
  see RFspectral.cc for spectral turning bands
@@ -37,7 +36,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 typedef struct tbm_lines{
   int lines;           // number of lines simulated
-  Real linesimufactor, /*factor by which simulation on the line is finer 
+  double linesimufactor, /*factor by which simulation on the line is finer 
 			 than the grid */
     linesimustep;      /* grid lag defined absolutely */
     int every; 
@@ -46,12 +45,10 @@ typedef struct tbm_lines{
 tbm_lines tbm2 =   { 60, 2.0, 0.0, 0};
 tbm_lines tbm3d2 = {500, 2.0, 0.0, 0};
 tbm_lines tbm3d3 = {500, 2.0, 0.0, 0};
+bool TBM2NUMERIC = true;
     
-#ifdef doubleReal 
-ce_param TBMCE = {false, true, TRIVIALSTARTEGY, -1e-7, 1e-3, 3, 0, 0, 0, 0};
-#else
-ce_param TBMCE = {false, true, TRIVIALSTARTEGY, -1e-5, 1e-3, 3, 0, 0, 0, 0};
-#endif
+ce_param TBMCE = {false, true, TRIVIALSTARTEGY, -1e-7, 1e-3, 3, 10000000.0, 
+		  0, 0, 0, 0};
 
 SimulationType TBM_METHOD=CircEmbed;
 
@@ -60,10 +57,10 @@ typedef struct TBM_storage {
 		          circulantembedding) */
   SimulationType method;
   int nn[2], m[2], halfm[2], nnhalf, cumm[3]; /*** !!! ***/
-  Real linesimuscale, half[MAXDIM], *x;
+  double linesimuscale, half[MAXDIM], *x;
   int mtot; 
   double *c,*d;
-  Real *simuline;
+  double *simuline;
   FFT_storage FFT;
   direct_storage *Direct;
   int simuspatialdim, timespacedim, ce_dim;
@@ -72,121 +69,116 @@ typedef struct TBM_storage {
 } TBM_storage;
 
 
-double TBM2_INTEGR_PREC=1e-9;
-#define NNN 36 /* immmer gerade !! */
-Real TBM2integr(int covnr, Real h, Real v, Real *param, int dim, int fct) {
-  Real de, ialt, ineu, x, y, z, factor[3] = {0.5, 2.0, 0.5}, 
-    xx[3], b[NNN+1], integral, delta, alpha, f, h2, v2, rest; 
+double TBM2_INTEGR_PREC=1e-10;
+#define NNN 40 /* immmer gerade !! */
+double TBM2integr(int covnr, double h, double u, double *param, int dim, int fct) {
+  // fct==0 : cov_tbm3 is coded, space & time separated
+  // fct==1 : cov_tbm3 is not coded, space & time separated
+  // fct==2 : isotropic function in space-time
+  double de, ialt, ineu, x, y, z[2], factor[3] = {0.5, 2.0, 0.5}, 
+    b[NNN+1], integral, delta,  f, h2, u2, alpha; 
   unsigned long int n, k;
-  int i, mitte, kk;
-    
-  h2 = h * h;
-  v2 = v * v;
-  b[0] = 0.0;
-  b[NNN] = 1 - 1e-13;
-  mitte = 1;
-  b[mitte] = 0.7;
-  alpha = 0.19;
-  for (i=mitte + 1; i<NNN ; i++) {
-    b[i] = exp(0.5 * log(alpha + (1.0 - alpha) * b[i-1]));
-  }
-  // alpha = 0.7;
-  for (i=mitte - 1; i>0 ; i--) {
-    b[i] = (double) i / (double) mitte * b[mitte];
-  }
+  int i, unten, oben, mitte, kk, zz //, zzcum
+    ;
 
+  //zzcum=0;
+  assert(fct>=0 && fct<=2);
+  h2 = h * h;
+  u2 = u * u;
+  z[1] = u;
+  unten = 4;
+  mitte = 12;
+  oben = 26;
+  b[unten] = 0.1;
+  b[mitte] = 0.8;
+  b[oben] = 0.95;
+  b[0] = 0.0;
+  b[NNN] = 1.0;
+  alpha = 0.2;
+  for (i=unten-1; i>0; i--)  b[i] = alpha * b[i+1] + (1.0 - alpha) * b[0];
+  for (i=unten+1; i<mitte; i++) 
+    b[i] = (double) (i - unten) / (double) (mitte - unten) *
+      (b[mitte] - b[unten]) + b[unten];
+  alpha = 0.4;
+  for (i=mitte+1; i<oben; i++) b[i] = alpha * b[i-1] + (1.0 - alpha) * b[oben];
+  alpha = 0.5;
+  for (i=oben+1; i<NNN; i++) b[i] = alpha * b[i-1] + (1.0 - alpha) * b[NNN];
   integral = 0.0;
   for (i=0; i<NNN; i++) {
-//    printf(" i=%d\n ",i);
-    n = 2;
+     n = 2;
     delta = b[i+1] - b[i];
     de = delta / (double) n;
     ialt = 0.0;
     for (k=0, x=b[i]; k<=2; k++, x+=de) {
       switch(fct) {
-	  case 0 : 
-	    z = h * x;
-	    f = CovList[covnr].cov_tbm3(&z, param, dim);
+	  case 0 : // cov_tbm3 coded, space & time separated
+	    z[0] = h * sqrt(1.0 - x * x);
+	    f = CovList[covnr].cov_tbm3(z, param, dim);
 	    break;
-	  case 1 :
-	    z = fabs(h * x);
-	    f = CovList[covnr].cov(&z, param, dim) +
-	      z * CovList[covnr].ableitung(&z, param, dim);
+	  case 1 : // cov_tbm3 not coded, space & time separated
+	    z[0] = h * sqrt(1.0 - x * x);
+	    f = CovList[covnr].cov(z, param, dim) +
+	      z[0] * CovList[covnr].derivative(z, param, dim);
 	    break;
-	  case 2 :
-	    y = x * x * h2;
-	    z = sqrt(y + v2); 
-	    if (z==0.0) f=1.0; 
-	    else f = CovList[covnr].cov(&z, param, dim) +
-	      y / z * CovList[covnr].ableitung(&z, param, dim);
+	  case 2 : // isotropic function in space-time
+	    assert(CovList[covnr].isotropic==FULLISOTROPIC);
+	    y = (1.0 - x * x) * h2;
+	    z[0] = sqrt(y + u2); 
+	    if (z[0]==0.0) f=1.0; 
+	    else f = CovList[covnr].cov(z, param, dim) +
+	      y / z[0] * CovList[covnr].derivative(z, param, dim);
 	    break;
-	  default : break;
+	  default : assert(false);
 	}
-      ialt += factor[k]  * x / sqrt(1 - x * x) * f;
-//  printf(" %d x=%f %f \n", n, x, ialt * h);
-      rest = f * sqrt(1 - b[NNN] * b[NNN]);
+      ialt += factor[k] * f;
     }
     
-    while (true) {
+    for (zz=0; zz<12; zz++) { // sonst rechnet der sich u.U. tot
       n *= 3;
       de = delta / (double) n;
       ineu = 0.0;
       for (k=1, x=b[i]+de; k<n; k++, x+=de) {
 	kk = k % 6;
 	if (kk == 0 || kk == 3) continue;
+
+	//zzcum++;
+
 	switch (fct) {
 	  case 0 : 
-	    z = h * x;
-//	    printf(" z=%f\n", z);
-	    f = CovList[covnr].cov_tbm3(&z, param, dim);
+	    z[0] = h * sqrt(1.0 - x * x);
+	    f = CovList[covnr].cov_tbm3(z, param, dim);
 	    break;
 	  case 1 :
-	    z = fabs(h * x);
-	    f = CovList[covnr].cov(&z, param, dim) +
-	      z * CovList[covnr].ableitung(&z, param, dim);
-//	    printf("%d %f zz=%f %f %f %f\n", n, x ,z, f, 
-//		   CovList[covnr].cov(&z, param, dim),
-//		   CovList[covnr].ableitung(&z, param, dim));
+	    z[0] = h * sqrt(1.0 - x * x);
+	    f = CovList[covnr].cov(z, param, dim) +
+	      z[0] * CovList[covnr].derivative(z, param, dim);
 	    break;
 	  case 2 :
-	    y = x * x * h2;
-	    z = sqrt(y + v2); 
-	    if (z==0.0) f=1.0; 
-	    else f = CovList[covnr].cov(&z, param, dim) +
-		   y / z * CovList[covnr].ableitung(&z, param, dim);
+	    y =  (1.0 - x * x) * h2;
+	    z[0] = sqrt(y + u2); 
+	    if (z[0]==0.0) f=1.0; 
+	    else f = CovList[covnr].cov(z, param, dim) +
+		   y / z[0] * CovList[covnr].derivative(z, param, dim);
 	    break;
-	  default : break;
+	  default : assert(false);
 	}
 	if (kk==1 || kk==5) f *= 2.0;
-	ineu += x / sqrt(1 - x * x) * f; 
-//	printf(" %f %f %f f=%f %f %f %f i=%f\n", 
-//	       x, y, z , f, CovList[covnr].cov(&z, param, dim),
-//	       y / z,  CovList[covnr].ableitung(&z, param, dim), ineu);
+	ineu += f; 
       }
-      if ((n>6) && (fabs(ineu - 2 * ialt) * de < TBM2_INTEGR_PREC)) break;
-if (n>1000) printf(" %d [%5.3e %5.3e] %e %e %e h=%e v=%e\n",
-		   n, 1-b[i], 1-b[i+1], ineu * de, ialt * de,
-		  fabs(ineu - 2 * ialt) * de, h, v);
-//     assert (n<=10000) ;
+      if ((fabs(ineu - 2 * ialt) * de < TBM2_INTEGR_PREC)) break;
       ialt += ineu;
-//   assert(false);
     }
-//    printf("\n %d [%5.3e %5.3e] %e; y=%f z=%f %e f=%f",
-//	   n, 1-b[i], 1-b[i+1], ialt * de / 1.5,
-//	   y, z,  fabs(ineu - 2 * ialt) * de, f);
-   ialt += ineu;
+    ialt += ineu;
    integral += ialt * de / 1.5;
   }
-//  assert(false);
-//printf("%d a=%f h=%f S=%f \n", n, aa, aa * c, integral);
-  return integral + rest;
+  return integral;
 }
 
-
+/*
 double TBM2_INTEGR_TIME_PREC=1e-9;
-Real TBM2integr_time(Real aa, Real c, covfct f, Real *param, int dim) {
-#define NN 26 /* immmer gerade !! */
-  Real h, ialt, ineu, bb=1.0, x, y, z, factor[3]= {0.5, 2.0, 0.5}, 
+double TBM2integr_time(double aa, double c, covfct f, double *param, int dim) {
+#define NN 26 // immmer gerade !! 
+  double h, ialt, ineu, bb=1.0, x, y, z, factor[3]= {0.5, 2.0, 0.5}, 
     xx[3], b[NN+1], integral, delta, alpha; 
   unsigned long int n,k;
   int i, mitte;
@@ -214,7 +206,6 @@ Real TBM2integr_time(Real aa, Real c, covfct f, Real *param, int dim) {
       y = x * x;
       z = c * sqrt(1.0 - y); 
       ialt += factor[k] * f(&z, param, dim) / y;
-//  printf(" %d x=%f %f \n", n, x, ialt * h);
     }
     
     while (true) {
@@ -237,33 +228,29 @@ Real TBM2integr_time(Real aa, Real c, covfct f, Real *param, int dim) {
 	}
       }
       if ((n>6) && (fabs(ineu - 2 * ialt) * h < TBM2_INTEGR_TIME_PREC)) break;
-//    printf(" %d %f \n", n, ineu * h);
       ialt += ineu;
-//    assert(false);
     }
-//    printf("\n %d [%f %f] %e; %f %f %f", n, b[i], b[i+1], ialt * h / 1.5,
-//	  y, z, f(&z, param, dim));
     ialt += ineu;
     integral += ialt * h / 1.5;
   }
-//printf("%d a=%f h=%f S=%f \n", n, aa, aa * c, integral);
   return integral;
 }
+*/
 
 
 #define ANISOP3 ANISO+3  
-Real CovFctTBM2(Real *x, int dim, int *covnr, int *op,
+double CovFctTBM2(double *x, int dim, int *covnr, int *op,
 	        param_type param, int ncov, bool anisotropy){
 // in dieser und der naechste Funktionen hat $x$ entweder
 // 1 Element (Geradenpunkt der turning bands) oder 2 Elemente
 // (Punkt auf Flaeche der turning layers)
-  int v, vold, w;
-  Real zw, result, r;
+  int v, vold;
+  double zw, result;
   result = 0.0;
   v = 0;
-
   if (dim==2) { // => s->anisotropy! (time-space simulation)
-    Real z[2]; //'' !! 2 !
+    int w;
+    double z[2]; //'' !! 2 !
     while (v<ncov) {
       vold = v;
       v++; while ((v<ncov) && op[v-1]) v++; 
@@ -272,43 +259,34 @@ Real CovFctTBM2(Real *x, int dim, int *covnr, int *op,
 	// there could be multiplication by pure time components (z[0]=0)!
 	z[0] = param[w][ANISO] * x[0];
 	z[1] = param[w][ANISOP3] * x[1];
-	// printf("z=%f %f\n", z[0], z[1]);
 	if (CovList[covnr[w]].isotropic==FULLISOTROPIC) {
 	  if (z[0]==0.0) {
 	    zw *= param[w][VARIANCE] *
 	      CovList[covnr[w]].cov(&(z[1]), param[w], dim); 
 	  } else { // called only once in the loop
-	    z[0] = fabs(z[0]);
-	    r = sqrt(z[0] * z[0] + z[1] * z[1]);
-	    zw *= param[w][VARIANCE] * z[0] / r *
-	      ( CovList[covnr[w]].cov_tbm2(&r, param[w], dim) +
-		TBM2integr_time(z[0] / r, r, CovList[covnr[w]].cov,
-				param[w], dim) );
-
-	    printf("\nh=%f v=%f, %f %f, c=%f %e %e\n", z[0], z[1], 
-	     CovList[covnr[w]].cov_tbm2(&r, param[w], dim)
-		   ,
-	     TBM2integr_time(z[0] / r, r, CovList[covnr[w]].cov, param[w], dim)
-		   ,
-	     CovList[covnr[w]].cov_tbm2(&r, param[w], dim) +
-	     TBM2integr_time(z[0] / r, r, CovList[covnr[w]].cov, param[w], dim)
-		   ,
-	     CovList[covnr[w]].cov_tbm2(&r, param[w], dim) +
-	     TBM2integr_time(z[0] / r, r, CovList[covnr[w]].cov, param[w], dim)
-		   - TBM2integr(covnr[w], z[0], z[1], param[w], dim, 2),
-		   TBM2integr(covnr[w], z[0], z[1], param[w], dim, 2)
-	);
+	    zw *= param[w][VARIANCE] * 
+	      TBM2integr(covnr[w], z[0], z[1], param[w], 2, 2);
 
+	    /* 
+               // alternatively: faster, but more complicated,
+	       // and uses cov_tbm2
+	       z[0] = fabs(z[0]);
+	       r = sqrt(z[0] * z[0] + z[1] * z[1]);
+	       zw *= param[w][VARIANCE] * z[0] / r *
+	       ( CovList[covnr[w]].cov_tbm2(&r, param[w], dim) +
+	       TBM2integr_time(z[0] / r, r, CovList[covnr[w]].cov,
+	       param[w], dim) );
+	    */
 	  }
 	} else { 
 	  assert(CovList[covnr[w]].isotropic==SPACEISOTROPIC);
 	  if (z[0]==0.0) 
-	    zw *=  CovList[covnr[w]].cov(z, param[w], 2);
-	  else if (CovList[covnr[w]].cov_tbm2 != NULL) {
+	    zw *= CovList[covnr[w]].cov(z, param[w], 2);
+	  else if (param[w][TBM2NUM] == 0.0) {
 	    zw *= CovList[covnr[w]].cov_tbm2(z, param[w],2);
-	   } else {
-	    assert(CovList[covnr[w]].ableitung != NULL);
-	    zw *= TBM2integr(covnr[w], z[0], 0, param[w], 2,
+	  } else {
+	    assert(CovList[covnr[w]].derivative != NULL);
+	    zw *= TBM2integr(covnr[w], z[0], z[1], param[w], 2,
 			     CovList[covnr[w]].cov_tbm3 == NULL);
 	  }
 	  zw *= param[w][VARIANCE];
@@ -318,34 +296,33 @@ Real CovFctTBM2(Real *x, int dim, int *covnr, int *op,
     }
   } else {
     assert(dim==1);
-    Real z;
-    for (v=0; v<ncov; v++){
+    double z;
+    for (v=0; v<ncov; v++) {
       // the field must be isotropic, so the x's have been transformed
       // multiplication of covariance functions is not allowed,
       // in contrast to 3dim TBM
-    z = x[0] * param[v][ANISO];
-
-    if (z==0.0) zw = 1.0;
-    else if (CovList[covnr[v]].cov_tbm2 != NULL) {
-      zw = CovList[covnr[v]].cov_tbm2(&z, param[v], dim);
-    } else {
-      assert(CovList[covnr[v]].ableitung != NULL);
-      zw = TBM2integr(covnr[v], z, 0, param[v], 2,
-		       CovList[covnr[v]].cov_tbm3 == NULL);
-    }
-    result += param[v][VARIANCE] * zw;
+      z = x[0] * param[v][ANISO];
+      if (z==0.0) zw = 1.0;
+      else if (param[v][TBM2NUM] == 0.0) {
+	zw = CovList[covnr[v]].cov_tbm2(&z, param[v], dim);
+      } else {
+	assert(CovList[covnr[v]].derivative != NULL);
+	zw = TBM2integr(covnr[v], z, 0, param[v], 2,
+			CovList[covnr[v]].cov_tbm3 == NULL);
+      }
+      result += param[v][VARIANCE] * zw;
     }
   }
   return result;
 }
 
 
-Real CovFctTBM3(Real *x, int dim, int *covnr, int *op,
+double CovFctTBM3(double *x, int dim, int *covnr, int *op,
 	        param_type param, int ncov, bool anisotropy){
   int v, vold, w, y;
-  Real dummy, zw, result;
-  Real r, z[2]; //'' !! 2 !
-  Real cov[MAXCOV], abl[MAXCOV];
+  double dummy, zw, result;
+  double r, z[2]; //'' !! 2 !
+  double cov[MAXCOV], abl[MAXCOV];
   result = 0.0;
   v = 0;
 
@@ -361,13 +338,13 @@ Real CovFctTBM3(Real *x, int dim, int *covnr, int *op,
 	  r = sqrt(z[0] * z[0] + z[1] * z[1]);
 	  if (r==0) abl[w] = 0;
 	  else abl[w] = param[w][VARIANCE] * param[w][ANISO] * fabs(z[0]) / r * 
-		 CovList[covnr[w]].ableitung(&r, param[w], dim);
+		 CovList[covnr[w]].derivative(&r, param[w], dim);
 	  zw *= (cov[w] =param[w][VARIANCE]*CovList[covnr[w]].cov(&r, param[w], 
 								dim));
 	} else { 
 	  assert(CovList[covnr[w]].isotropic==SPACEISOTROPIC);
 	  abl[w] = param[w][VARIANCE] * 
-	    param[w][ANISO] * CovList[covnr[w]].ableitung(z, param[w], dim);
+	    param[w][ANISO] * CovList[covnr[w]].derivative(z, param[w], dim);
 	  zw *= (cov[w] =param[w][VARIANCE]*CovList[covnr[w]].cov(z, param[w], 
 								dim));
 	}
@@ -388,19 +365,19 @@ Real CovFctTBM3(Real *x, int dim, int *covnr, int *op,
   } else {
     assert(dim==1);
     z[1] = 0.0; //in case of (CovList[covnr[v]].isotropic==SPACESOTROPIC) 
-     if (ncov==1) {
-     z[0] = x[0] * param[0][ANISO];
-     return param[v][VARIANCE]*CovList[covnr[0]].cov_tbm3(z, param[0], dim);
+    if (ncov==1 && CovList[covnr[0]].cov_tbm3!=NULL) {
+      z[0] = x[0] * param[0][ANISO];
+      return param[v][VARIANCE]*CovList[covnr[0]].cov_tbm3(z, param[0], dim);
     }
     while (v<ncov) {
       vold = v;
       v++; while ((v<ncov) && op[v-1]) v++;
       zw = 1.0;
       for (w=vold; w<v; w++) {
-	z[0] = x[0] * param[w][ANISO];
+	z[0] = fabs(x[0] * param[w][ANISO]);
 	zw *= (cov[w] = 
 	       param[w][VARIANCE] * CovList[covnr[w]].cov(z, param[w], dim)); 
- 	abl[w] = param[w][VARIANCE]*CovList[covnr[w]].ableitung(z, param[w],
+ 	abl[w] = param[w][VARIANCE]*CovList[covnr[w]].derivative(z, param[w],
 								dim);
       }
       result += zw;
@@ -418,7 +395,7 @@ Real CovFctTBM3(Real *x, int dim, int *covnr, int *op,
   return result;
 }
 
-void do_tbmcirculantembedding(key_type *key, int m, Real *res ) {
+void do_tbmcirculantembedding(key_type *key, int m, double *res ) {
   TBM_storage *s;
   s = (TBM_storage*) key->S[m];
   internal_do_circ_embed(s->nn, s->m, s->cumm, s->halfm,
@@ -426,7 +403,7 @@ void do_tbmcirculantembedding(key_type *key, int m, Real *res ) {
 			 &(s->FFT), false, res );
 }
 
-void do_tbmdirect(key_type *key, int m, Real *res ) {
+void do_tbmdirect(key_type *key, int m, double *res ) {
   TBM_storage *s;
   s = (TBM_storage*) key->S[m];
   internal_do_directGauss(s->Direct, false, s->mtot, res);
@@ -448,16 +425,17 @@ void TBM_destruct(void **S)
   }
 }
 
-void SetParamTBMCE( int *action, int *force, Real *tolRe, Real *tolIm,
-			int *trials, int *mmin, int *userfft, int *strategy) 
+void SetParamTBMCE( int *action, int *force, double *tolRe, double *tolIm,
+			int *trials, int *mmin, int *userfft, int *strategy,
+			double *maxmem) 
 {
   SetParamCE(action, force, tolRe, tolIm, trials, mmin, userfft, strategy,
-	     &TBMCE,"TBMCE");
+	      maxmem, &TBMCE,"TBMCE");
 }
 
 
-void SetParamLines(int *action,int *nLines, Real *linesimufactor, 
-		  Real *linesimustep, tbm_lines *tbm, int *every, char *name) {
+void SetParamLines(int *action,int *nLines, double *linesimufactor, 
+		  double *linesimustep, tbm_lines *tbm, int *every, char *name) {
   switch (*action) {
   case 0 :
     tbm->lines=*nLines;
@@ -484,20 +462,27 @@ void SetParamLines(int *action,int *nLines, Real *linesimufactor,
   }
 }
 
-void SetParamTBM2(int *action,int *nLines, Real *linesimufactor, 
-		  Real *linesimustep, int *every) {
+void SetParamTBM2(int *action,int *nLines, double *linesimufactor, 
+		  double *linesimustep, int *every, int *tbm2num) {
   SetParamLines(action, nLines, linesimufactor, linesimustep, &tbm2, every,
 		"TBM2");
+  switch (*action) {
+  case 0 : TBM2NUMERIC = (bool) *tbm2num; break;
+  case 1 : *tbm2num = TBM2NUMERIC; break;
+  case 2 : PRINTF("\n%s\n====\nTBM2 numerical covariance=%d", TBM2NUMERIC);
+    break;
+  default : PRINTF(" unknown action\n"); 
+  }
 }
 
-void SetParamTBM3D2(int *action,int *nLines, Real *linesimufactor, 
-		  Real *linesimustep, int *every) {
+void SetParamTBM3D2(int *action,int *nLines, double *linesimufactor, 
+		  double *linesimustep, int *every) {
   SetParamLines(action, nLines, linesimufactor, linesimustep, &tbm3d2, every,
 		"TBM3D2");
 }
 
-void SetParamTBM3D3(int *action,int *nLines, Real *linesimufactor, 
-		  Real *linesimustep, int *every) {
+void SetParamTBM3D3(int *action,int *nLines, double *linesimufactor, 
+		  double *linesimustep, int *every) {
   SetParamLines(action, nLines, linesimufactor, linesimustep, &tbm3d3, every,
 		"TBM3D3");
 }
@@ -537,18 +522,22 @@ void SetParamTBM(int *action, int *tbm_method) {
 int init_turningbands(key_type *key, SimulationType method, int m)
 {
   param_type param;
-  int error, i, d, tbm_dim, v, start_aniso[MAXDIM], index_dim[MAXDIM];
+  int error, i, d, tbm_dim, v, start_aniso[MAXDIM], index_dim[MAXDIM],
+      lasttimecomponent, firsttimecomponent;
   CovFctType CovFctTBM;
   TBM_storage *s;
-  bool no_last_comp;
-  Real dist, mindelta, aniso_last_save, 
+  bool no_last_comp, storing;
+  double dist, mindelta, aniso_last_save, 
     quotient[MAXCOV], *directx[2];
   unsigned short int actcov;
   int covnr[MAXCOV], multiply[MAXCOV], nonzero_pos;
   tbm_lines * tbm;
   SimulationType tbm_method;
-  
 
+  lasttimecomponent = key->totalparam - 1;
+  firsttimecomponent = key->totalparam - key->timespacedim;
+           /* multiplication of the anisotropy matrix from the right */
+  storing = GENERAL_STORING;
   GENERAL_STORING = true;
   nonzero_pos = -1;
   SET_DESTRUCT(TBM_destruct);
@@ -595,10 +584,10 @@ int init_turningbands(key_type *key, SimulationType method, int m)
    submatrix for v=0, already checked by FIRST_CHECK_
     */
 
-    if (GENERAL_PRINTLEVEL>4) PRINTF("\nextracting matching covariances...");
-    Real store_param[TOTAL_PARAM];
+    if (GENERAL_PRINTLEVEL>4) PRINTF("extracting matching covariances...");
+    double store_param[TOTAL_PARAM];
     long bytes;
-    bytes = sizeof(Real) * key->timespacedim * key->timespacedim;
+    bytes = sizeof(double) * key->timespacedim * key->timespacedim;
     actcov=0;
     int v;
     for (v=0; v<key->ncov; v++) {
@@ -608,7 +597,7 @@ int init_turningbands(key_type *key, SimulationType method, int m)
 	assert((key->covnr[v]>=0) && (key->covnr[v]<currentNrCov));
 	assert(key->param[v][VARIANCE]>=0.0);
 	covnr[actcov] = key->covnr[v];
-	memcpy(param[actcov], key->param[v], sizeof(Real) * key->totalparam);
+	memcpy(param[actcov], key->param[v], sizeof(double) * key->totalparam);
 	if (actcov>0) {
 	  if ((multiply[actcov-1] = key->op[v-1]) && key->method[v-1]!=method) {
 	    if (GENERAL_PRINTLEVEL>0) 
@@ -623,7 +612,7 @@ int init_turningbands(key_type *key, SimulationType method, int m)
 	  if (key->Time) endfor -= key->timespacedim; // note: time component
 	  //   of the anisotropy matrix is not checked, which has the form
 	  // (0 0 0 *)^T
-	  Real f_epsilon = 1e-15;
+	  double f_epsilon = 1e-15;
 	  if (actcov>0) {
 	    /* check whether the multiplicative construction are consistent,*/
 	    /* i.e. that the SPATIAL part of the anisotropy matrices are    */
@@ -665,15 +654,15 @@ int init_turningbands(key_type *key, SimulationType method, int m)
 	    if (nonzero_pos>=endfor) { error=ERRORTRIVIAL; goto ErrorHandling; }
 	  }
 	} else {
-	  assert(fabs(param[v][SCALE] * param[v][INVSCALE]-1.0) < EPSILON);
+	  assert(fabs(param[actcov][SCALE] * param[actcov][INVSCALE]-1.0) < 
+		 EPSILON);
 	  quotient[actcov] = 1.0;
 	}
 	actcov++;
       } // v
     }
     if (actcov==0) { /* no covariance for the considered method found */
-      if (key->traditional) error=ERRORNOTINITIALIZED;
-      else error=NOERROR_ENDOFLIST;
+      error=NOERROR_ENDOFLIST;
       goto ErrorHandling;
     }
   }
@@ -687,12 +676,12 @@ int init_turningbands(key_type *key, SimulationType method, int m)
   if (GENERAL_PRINTLEVEL>4) PRINTF("\nchecking parameter structure...");
 
   // in GetTrueDim only the first anisotropy matrix is investigated
-  // so if there is a time component [key->totalparam-1], it 
+  // so if there is a time component at lastimecomponent, it 
   // is written temporarily into the first matrix
-  aniso_last_save = param[0][key->totalparam-1]; 
+  aniso_last_save = param[0][lasttimecomponent]; 
   for (v=0; v<actcov; v++) 
-    if (param[v][key->totalparam-1]!=0.0) {
-      param[0][key->totalparam-1]=1.0;
+    if (param[v][lasttimecomponent]!=0.0) {
+      param[0][lasttimecomponent]=1.0;
       break; 
     }
   // no_last_comp: last dim of aniso matrix is identically zero
@@ -702,7 +691,7 @@ int init_turningbands(key_type *key, SimulationType method, int m)
 	     &s->timespacedim,
 	     &no_last_comp, // vanishing for *all* v=0..actcov-1 ?! 
 	     start_aniso, index_dim);
-  param[0][key->totalparam-1] = aniso_last_save;
+  param[0][lasttimecomponent] = aniso_last_save;
 
   s->simuspatialdim = s->timespacedim;
   if (key->Time && !no_last_comp)  // note: no_last_comp means for all 0..actcov
@@ -732,15 +721,12 @@ int init_turningbands(key_type *key, SimulationType method, int m)
     TimeSpaceMix = false;
     if (twodim_ce = key->Time && !no_last_comp) {
       // note: no_last_comp means for all 0..actcov
-      int start, endfor;
-      start  = key->totalparam - key->timespacedim;
-      endfor = key->totalparam-1;
       for (v=0; v<actcov; v++) {
 	/* is it of the form   * * 0
 	   .                   * * 0       ?
            .                   * * x 
 	*/	
-	for (w=start; w<endfor; w++) {
+	for (w=firsttimecomponent; w<lasttimecomponent; w++) {
 	  if (TimeSpaceMix = param[v][w]!=0.0) {
 	    error = ERRORTIMESEPARATE;
 	    goto ErrorHandling;
@@ -763,30 +749,26 @@ int init_turningbands(key_type *key, SimulationType method, int m)
   for (v=0; v<actcov; v++) {
     cov_fct *cov;
     cov = &(CovList[covnr[v]]);
-    if (((method==TBM3) && (cov->ableitung==NULL)) ||
-	((method==TBM2) && 
-	 ((cov->cov_tbm2==NULL && cov->ableitung==NULL && !key->anisotropy) ||
-	  (key->anisotropy && 
-	   (((param[v][nonzero_pos]!=0.0) && cov->cov_tbm2==NULL && 
-	     cov->ableitung==NULL) ||
-	    ((param[v][nonzero_pos]==0.0) && cov->cov==NULL)
-	    )
-	    )
-	  )
-	  )
-      )
-      { 
-	error=ERRORNOTDEFINED; goto ErrorHandling;}
+    if (cov->implemented[method] <= NOT_IMPLEMENTED) { 
+      error = ERRORNOTDEFINED; goto ErrorHandling;}
     if ((!key->anisotropy && (CovList[covnr[v]].isotropic!=FULLISOTROPIC)) ||
 	(cov->isotropic==ANISOTROPIC) ||
 	((cov->isotropic==SPACEISOTROPIC) && (s->ce_dim==1))
-      ) { error= ERRORISOTROPICMETHOD; goto ErrorHandling;}
+      ) { error = ERRORISOTROPICMETHOD; goto ErrorHandling;}
     if ((key->Time) && no_last_comp && (cov->isotropic!=FULLISOTROPIC)) {
       error = ERRORWITHOUTTIME; goto ErrorHandling;} // braucht's net falls
     // GetDim und Trans2nogrid die dimensionen belassen!!
-    if ((cov->check!=NULL) && 
-	((error=cov->check(param[v], s->timespacedim, method))) != NOERROR)
+ 
+    if (cov->check!=NULL)
+      error=cov->check(param[v], s->timespacedim, method);
+    if (error && (error!=ERRORCOVNUMERICAL || method!=TBM2 || !TBM2NUMERIC))
       goto ErrorHandling;
+    param[v][TBM2NUM] =
+      (double) ((error && cov->implemented[TBM2] >= IMPLEMENTED) ||
+		(cov->implemented[TBM2]==NUM_APPROX && TBM2NUMERIC));
+
+    if (method==TBM2 && GENERAL_PRINTLEVEL > 1 && (param[v][TBM2NUM]!=0.0))
+      PRINTF("\tnumerical evaluation of the TBM operator for %s\n", cov->name);
   }
 
   if ((s->ce_dim==2) && (method==TBM2)) {
@@ -844,10 +826,10 @@ int init_turningbands(key_type *key, SimulationType method, int m)
     assert(CovList[covnr[0]].isotropic==FULLISOTROPIC);
     {
       // diameter of the grid
-      register Real dummy;
+      register double dummy;
       dist = 0.0; // max distance
       for (d=0; d < s->simuspatialdim; d++) {
-	dummy = key->x[d][XSTEP] * (Real) (key->length[d] - 1);
+	dummy = key->x[d][XSTEP] * (double) (key->length[d] - 1);
 	dist += dummy * dummy;
       }
     }
@@ -870,11 +852,11 @@ int init_turningbands(key_type *key, SimulationType method, int m)
     }
   } else { // not s->grid
     // s->timespacedim, s->x, s->spatialdim,
-    Real min[MAXDIM],max[MAXDIM], dummy; 
+    double min[MAXDIM],max[MAXDIM], dummy; 
     int j, d, ix, jx, endfor;
     for (d=0; d<MAXDIM; d++) {min[d]=RF_INF; max[d]=RF_NEGINF;}
     if (key->grid) { // key->grid     
-      Real sxx[ZWEIHOCHMAXDIM * MAXDIM];
+      double sxx[ZWEIHOCHMAXDIM * MAXDIM];
       // unsorted, reduced for param[0...0], #=2^Dim, 
       endfor = 1 << key->timespacedim;
       
@@ -910,7 +892,7 @@ int init_turningbands(key_type *key, SimulationType method, int m)
 	if (tbm->linesimustep==0.0) {
 	  // determine smallest distance ((may take a lot of time!!))
 	  for (jx=j=0; j<i; j++, jx+=s->timespacedim) {
-	    register Real diff,dist;
+	    register double diff,dist;
 	    for(dist=0.0, d=0; d<s->timespacedim; d++) {
 	      diff = s->x[ix+d] - s->x[jx+d]; 
 	      dist += diff * diff;
@@ -937,7 +919,7 @@ int init_turningbands(key_type *key, SimulationType method, int m)
     if (s->ce_dim==2) 
       for (v=0; v<actcov; v++) {
 	param[v][ANISO+1] = param[v][ANISO+2] = 0;
-	param[v][ANISOP3] = param[v][key->totalparam - 1] * key->T[XSTEP];
+	param[v][ANISOP3] = param[v][lasttimecomponent] * key->T[XSTEP];
       }
 
     if (false) 
@@ -962,7 +944,7 @@ int init_turningbands(key_type *key, SimulationType method, int m)
     // nn=number of pixels to be simulated on the lines [==(length of diagonal 
     // through grid that is to be simulated) * (linesimufactor=="precision")]
     // 5 is for safety
-    Real dummy;
+    double dummy;
     dummy = (sqrt(dist) * s->linesimuscale);
     if (dummy > MAXNN) {error=ERRORNN; goto ErrorHandling;}
     s->nn[0]= 5 + (long) dummy; // +5 for safety
@@ -993,16 +975,19 @@ int init_turningbands(key_type *key, SimulationType method, int m)
   tbm_method = key->tbm_method;
   if (s->mtot > DIRECTGAUSS_MAXVARIABLES) {
     tbm_method=CircEmbed;
-    if (GENERAL_PRINTLEVEL>2) 
-      PRINTF("tbm: too many points -- using circulant embedding instead of 'direct'\n");
+    if (GENERAL_PRINTLEVEL>1) 
+           PRINTF("tbm: too many points -- using circulant embedding instead of 'direct'\n");
   }
+
+//  printf("method: %d %d %d\n", tbm_method, Direct, CircEmbed);
+
   switch (tbm_method) {
       case Direct :
 	if (GENERAL_PRINTLEVEL>4)
 	  PRINTF("\ninitializing matrix decomposition...");
 	s->linemethod = do_tbmdirect;
 	for (d=0; d<2; d++) {
-	  if ((directx[d]= (Real *) malloc(sizeof(Real) * 3))==0) {
+	  if ((directx[d]= (double *) malloc(sizeof(double) * 3))==0) {
 	    error=ERRORMEMORYALLOCATION; goto ErrorHandling;
 	  }
 	  directx[d][0]=1.0; 
@@ -1023,11 +1008,15 @@ int init_turningbands(key_type *key, SimulationType method, int m)
 					actcov,
 					s->ce_dim>=2 // anisotropy
 	  );
+	if (error==NOERROR) break;
+/*
 	if (error!=NOERROR) goto ErrorHandling;
         // waere auch ein durchschleusen zu circembed moeglich
 	// koennte dann aber probleme schaffen, falls weitere Methoden
 	// hinzukommen.
 	break;
+*/
+
       case CircEmbed : 
 	long twoRealmtot;
 	if (GENERAL_PRINTLEVEL>4)
@@ -1052,7 +1041,7 @@ int init_turningbands(key_type *key, SimulationType method, int m)
 	break;
       default : error=ERRORNOTPROGRAMMED; goto ErrorHandling;
   }
-  if ((s->simuline=(Real *)malloc(sizeof(Real) * s->mtot))==0){
+  if ((s->simuline=(double *)malloc(sizeof(double) * s->mtot))==0){
     error=ERRORMEMORYALLOCATION;goto ErrorHandling;}
 
   if (GENERAL_PRINTLEVEL>4) PRINTF("\ntbm is now initialized.\n");
@@ -1060,7 +1049,8 @@ int init_turningbands(key_type *key, SimulationType method, int m)
   if (directx[0]!=NULL) free(directx[0]);
   if (directx[1]!=NULL) free(directx[1]);
 
-  // im folgenden anisotropy und nicht traditional, da 
+  GENERAL_STORING = storing;
+  // im folgenden unterscheidung bzgl. anisotropy, da 
   // oben alle Kovarianzfunktionen zusammengefasst werden mit derselben
   // (An)isotropie-Struktur, hei3t nur bei echter Anisotropie wird TBM2
   // etc. mehrfach aufgerufen!
@@ -1068,8 +1058,10 @@ int init_turningbands(key_type *key, SimulationType method, int m)
   else return NOERROR;
   
  ErrorHandling:
+  key->active = false;
   if (directx[0]!=NULL) free(directx[0]);
   if (directx[1]!=NULL) free(directx[1]);
+  GENERAL_STORING = storing;
   return error;
 }
 
@@ -1081,10 +1073,10 @@ int init_turningbands3(key_type *key, int m) {
   return(init_turningbands(key, TBM3, m));
 }
 
-void do_turningbands(key_type *key, int m, Real *res )
+void do_turningbands(key_type *key, int m, double *res )
 { 
-  Real phi;
-  Real *simuline;
+  double phi;
+  double *simuline;
   long n,nn, nnhalf; 
   simu1dim simulate;
   assert(key->active);
@@ -1116,21 +1108,21 @@ void do_turningbands(key_type *key, int m, Real *res )
   }
   
   if (s->grid) { // old form, isotropic field
-    Real deltax, yoffset,deltay, nxhalf, nyhalf,xoffset,
+    double deltax, yoffset,deltay, nxhalf, nyhalf,xoffset,
       linesimuscaleX,linesimuscaleY,linesimuscaleZ;  
     int nx,zaehler,ny;
-    nxhalf=((Real)(key->length[0]-1))/2.0; ////
-    nyhalf=((Real)(key->length[1]-1))/2.0; ////
+    nxhalf=((double)(key->length[0]-1))/2.0; ////
+    nyhalf=((double)(key->length[1]-1))/2.0; ////
     linesimuscaleX = s->linesimuscale * key->x[0][XSTEP]; ////
     if (s->method==TBM2) {// isotropy, TBM2, grid 
-      Real deltaphi;
-      deltaphi = PI / (Real) tbm.lines;
+      double deltaphi;
+      deltaphi = PI / (double) tbm.lines;
       phi = deltaphi * UNIFORM_RANDOM; 
       if (key->spatialdim==1) { // dim =1
 	for (n=0;n<tbm.lines;n++){
 	  if (tbm.every>0 && (n % tbm.every == 0)) PRINTF("%d \n",n);
 	  deltax=sin(phi) * linesimuscaleX;
-	  simulate(key, m, simuline );
+	  simulate(key, m, simuline);
 	  xoffset= (double)nnhalf - nxhalf * deltax;
 	  zaehler = 0;
 	  for(nx=0; nx<key->length[0];  nx++) {
@@ -1146,13 +1138,13 @@ void do_turningbands(key_type *key, int m, Real *res )
 	  if (tbm.every>0  && (n % tbm.every == 0)) PRINTF("%d \n",n);
 	  deltax=sin(phi) * linesimuscaleX;
 	  deltay=cos(phi) * linesimuscaleY;
-	  simulate(key, m, simuline );
+	  simulate(key, m, simuline);
 	  /* depending on deltax and deltay,
 	     shorter lines might be 
 	     simulated! -> improvement in 
 	     speed?! */
 	  /* then nnhalf must be kept variable, too: */
-	  yoffset= (double)nnhalf - nyhalf * deltay - nxhalf * deltax;
+	  yoffset= (double) nnhalf - nyhalf * deltay - nxhalf * deltax;
 	  zaehler = 0;
 	  for (ny=0;ny<key->length[1]; ny++) {	  
 	    xoffset = yoffset;
@@ -1173,7 +1165,7 @@ void do_turningbands(key_type *key, int m, Real *res )
 	    for (n=0;n<tbm.lines;n++){
 	      if (tbm.every>0  && (n % tbm.every == 0)) PRINTF("%d \n",n);
 	      deltax= UNIFORM_RANDOM * linesimuscaleX;
-	      simulate(key, m, simuline ); 
+	      simulate(key, m, simuline); 
 	      xoffset=  (double)nnhalf - nxhalf * deltax ;
 	      zaehler = 0;
 	      for(nx=0;nx<key->length[0];nx++) {
@@ -1187,15 +1179,15 @@ void do_turningbands(key_type *key, int m, Real *res )
 	    break;
 	  case 2:
 	    linesimuscaleY = s->linesimuscale * key->x[1][XSTEP]; ////
-	    for (n=0;n<tbm.lines;n++){
+	    for (n=0;n<tbm.lines;n++) {
 	      if (tbm.every>0  && (n % tbm.every == 0)) 
 		PRINTF("%d \n",n);
 	      deltax= UNIFORM_RANDOM;// see Martin's tech rep for details
 	      deltay= sqrt(1.0-deltax*deltax) * sin(UNIFORM_RANDOM*TWOPI) * 
 		linesimuscaleY;
 	      deltax *= linesimuscaleX;
-	      simulate(key, m, simuline ); 
-	      /* depending on deltax and deltay, shorter lines might be 
+ 	      simulate(key, m, simuline); 
+ 	      /* depending on deltax and deltay, shorter lines might be 
 		 simulated! -> improvement in speed!
 	      */
 	      yoffset=  (double)nnhalf - nyhalf * deltay - nxhalf * deltax ;
@@ -1215,11 +1207,11 @@ void do_turningbands(key_type *key, int m, Real *res )
 	    break;
 	  case 3:
 	   
-	    Real dummy,nzhalf,zoffset,deltaz;
+	    double dummy,nzhalf,zoffset,deltaz;
 	    int nz;
 	    linesimuscaleY = s->linesimuscale * key->x[1][XSTEP]; ////
 	    linesimuscaleZ= s->linesimuscale * key->x[2][XSTEP];
-	    nzhalf=((Real)(key->length[2]-1))/2.0;
+	    nzhalf=((double)(key->length[2]-1))/2.0;
 	    for (n=0; n<tbm.lines; n++){
 	      if (tbm.every>0  && (n % tbm.every == 0)) PRINTF("%d \n",n);
 	      //
@@ -1227,9 +1219,9 @@ void do_turningbands(key_type *key, int m, Real *res )
 	      dummy = sqrt(1.0 - deltaz * deltaz);
 	      deltaz *= linesimuscaleZ;
 	      //
-	      deltay= UNIFORM_RANDOM * TWOPI;
-	      deltax= cos(deltay) * dummy * linesimuscaleX;
-	      deltay= sin(deltay) * dummy * linesimuscaleY;
+	      deltay = UNIFORM_RANDOM * TWOPI;
+	      deltax = cos(deltay) * dummy * linesimuscaleX;
+	      deltay = sin(deltay) * dummy * linesimuscaleY;
 	      simulate(key, m, simuline );
 	      zoffset= (double) nnhalf - nzhalf * deltaz
 		- nyhalf * deltay - nxhalf * deltax; 
@@ -1266,8 +1258,6 @@ void do_turningbands(key_type *key, int m, Real *res )
   } else { 
     // not grid, could be time-model!
     // both old and new form included
-    //offset=0; for (i=0; i<s->mtot; i++) offset += simuline[i] * 
-    //                simuline[i];printf("%f ", offset / (double) s->mtot);
 
 #define TBMLOOP(UNITYVECTOR, OFFSET, INDEX)\
   {\
@@ -1275,16 +1265,29 @@ void do_turningbands(key_type *key, int m, Real *res )
       if (tbm.every>0  && (n % tbm.every == 0)) PRINTF("%d \n",n); \
       UNITYVECTOR\
       simulate(key, m, simuline); \
-      offset=(Real) nnhalf - (OFFSET); \
+      offset=(double) nnhalf - (OFFSET); \
       for (v = 0, i = 0; i < key->totalpoints; i++) {\
 	register long index;\
 	index = (long) (offset + INDEX);\
+/* if (v==0)\
+	  PRINTF("totpts=%d i=%d; index=%d mtot=%d n0=%d n1=%d offset=%f \nx=%f ex=%f halfx=%f \ny=%f ey=%f halfy=%f\nz=%f ez=%f halfz=%f\n t=%f scale=%f nnhalf=%d v=%d, i=%d\n", \
+		 key->totalpoints, i, \
+		 index,  s->mtot, s->nn[0], s->nn[1], offset,\
+		 s->x[v], ex, halfx,\
+		 s->x[v+1], ey, halfy,\
+		 s->x[v+2], ez, halfz,\
+                s->x[v+3], linescale, nnhalf,\
+		v, i); */ \
 	if ((index>=s->mtot) || (index<0)) {\
-	  PRINTF(" index=%d n0=%d n1=%d mtot=%d index=%f; offset=%f x=%f ex=%f t=%f scale=%f half=%f nnhalf=%d v=%d, i=%d\n", \
-		 index, s->nn[0], s->nn[1], s->mtot, offset + s->x[v] * ex,\
-		 offset, s->x[v], ex, s->x[v+1], linescale, halfx, nnhalf,\
+	  PRINTF("totpts=%d i=%d; index=%d mtot=%d n0=%d n1=%d offset=%f \nx=%f ex=%f halfx=%f \ny=%f ey=%f halfy=%f\nz=%f ez=%f halfz=%f\n t=%f scale=%f nnhalf=%d v=%d, i=%d\n", \
+		 key->totalpoints, i, \
+		 index,  s->mtot, s->nn[0], s->nn[1], offset,\
+		 s->x[v], ex, halfx,\
+		 s->x[v+1], ey, halfy,\
+		 s->x[v+2], ez, halfz,\
+                 s->x[v+3], linescale, nnhalf,\
                  v, i);\
-	  assert((index<s->mtot) &&  (index>=0));  \
+	  assert((index<s->mtot) &&  (index>=0));\
 	}\
 	res[i] += simuline[index];\
 	v += s->timespacedim;\
@@ -1297,7 +1300,7 @@ void do_turningbands(key_type *key, int m, Real *res )
   else {assert(s->ce_dim==2); TBMLOOP(UNITYVECTOR, OFFSET, INDEX + TIMEINDEX)};
 
 
-    Real halfx,halfy,ex,ey,offset,linescale;
+    double halfx=0,halfy=0, halfz=0,ex=0,ey=0,ez=0, offset,linescale;
     long v;
 
     halfx = s->half[0];
@@ -1305,8 +1308,8 @@ void do_turningbands(key_type *key, int m, Real *res )
     linescale = s->linesimuscale;
     int i;
     if (s->method==TBM2) {
-      Real deltaphi;
-      deltaphi = PI / (Real) tbm.lines;
+      double deltaphi;
+      deltaphi = PI / (double) tbm.lines;
       phi = deltaphi * UNIFORM_RANDOM; 
       if (s->simuspatialdim==1) { // dim == 1, TBM 2, arbitrary locations
 	TBMST({phi += deltaphi;ex=sin(phi) * linescale;}, //UNITYVECTOR
@@ -1320,9 +1323,10 @@ void do_turningbands(key_type *key, int m, Real *res )
 	      s->x[v+2]);
       }
     } else { // TBM3, not grid, dim=1 or 2
-      Real linesimusq, dummy, ez, halfz;
+      double linesimusq, dummy;
       assert(s->method==TBM3);
       linesimusq = linescale * linescale;
+      halfz = s->half[2];
       switch (s->simuspatialdim) {
 	  case 1 : //see Martin's techrep f. details
 	    TBMST({ex = UNIFORM_RANDOM * linescale;},
@@ -1341,7 +1345,6 @@ void do_turningbands(key_type *key, int m, Real *res )
 		  s->x[v+2]);
 	    break;
 	  case 3:
-	    halfz = s->half[2];
 	    TBMST({ez = UNIFORM_RANDOM * linescale;
 	           dummy = sqrt(linesimusq - ez * ez);
 		   ey = UNIFORM_RANDOM * TWOPI;
@@ -1357,8 +1360,8 @@ void do_turningbands(key_type *key, int m, Real *res )
   }
   {
     register long i;
-    register Real InvSqrtNlines;   
-    InvSqrtNlines=1.0 / sqrt((Real) tbm.lines);
+    register double InvSqrtNlines;   
+    InvSqrtNlines=1.0 / sqrt((double) tbm.lines);
     for(i=0;i<key->totalpoints;i++) { 
       res[i] *= InvSqrtNlines; 
     }
@@ -1369,8 +1372,10 @@ void do_turningbands(key_type *key, int m, Real *res )
 void pokeTBM(int *out, int *in, int *err) {
   int m;
   key_type *keyOut, *keyIn;
+  double *x;
   TBM_storage *sOut, *sIn;
   
+  keyIn = keyOut = NULL;
   *err = 0;
   if ((*out<0) || (*out>=MAXKEYS) || (*in<0) || (*in>=MAXKEYS))
     {*err=ERRORKEYNROUTOFRANGE; goto ErrorHandling;}
@@ -1399,20 +1404,22 @@ void pokeTBM(int *out, int *in, int *err) {
 	{*err=ERRORPOKETBMparam; goto ErrorHandling;}
       // other parameters should also be tested extensively!!
       grid = sIn->grid;
+      x = sIn->x;
+      keyIn -> S[m] = NULL;
       TBM_destruct(&(keyIn->S[m]));
       keyIn->S[m] = keyOut->S[m];
       keyOut->S[m]= NULL;
-      sIn = (TBM_storage*) keyIn->S[m];
+      sIn = (TBM_storage*) keyIn->S[m];     
       sIn->grid = grid;
+      sIn->x = x;
      }
   }
-  DeleteKey(out);
+  keyOut->active = false;
   return;
   
  ErrorHandling:
   if (GENERAL_PRINTLEVEL>3) PRINTF("error %d %d..\n", *in, *out);
-  DeleteKey(in);
-  DeleteKey(out);
+  keyIn->active = keyOut->active = false;
   return;
 }
 
